<?php

use Illuminate\Database\Capsule\Manager as Capsule;

class Cloudns_Controller {
	
	/**
	 * @var core 
	 */
	protected $core;
	protected $params;
	
	public function __construct ($params) {
		$this->core = Cloudns_Core::inst($params);
		$this->params = $params;
	}
	
	/**
	 * 
	 * @param string $zone
	 * @param string $zoneType
	 * @param int $option 1 or 0, with NS records or without records
	 * @param array $ns The servers to be used for NS records
	 * @param string $master_ip
	 * @return array
	 */
	public function addNewZone ($zone, $zoneType, $option, $ns, $master_ip) {
		
		try {
			$usersZones = $this->countUsersZones($this->params['serviceid']);
			if ($usersZones >= $this->params['zonesLimit']) {
				return array('status'=>'error','description'=>'You reached your zones limit of '.$this->params['zonesLimit'].', having '.$usersZones.' zones and can\'t add new zones.');
			}
			
			if ($zoneType == 'masterZoneType' || $zoneType == 'masterReverseZoneType') {
				$response = $this->core->Zones->addMaster($zone, $option, $ns);
			} else {
				$response = $this->core->Zones->addSlave($zone, $master_ip);
			}
			
			if ($response['status'] == 'success') {
				$zoneInfo = $this->core->Zones->getZoneInfo($zone);
				if (!$this->checkForFreeZone($zoneInfo['name'])) {
					return array('status'=>'error','description'=>'The zone was already added by you or another customer!');
				}
				
				$tbl_params = array(
					'serviceid' => $this->params['serviceid'],
					'name' => $zoneInfo['name'],
				);
				if (!$this->core->Database->insert('mod_cloudns_zones', $tbl_params)) {
					return array('status'=>'error','description'=>'The zone was not added!');
				}
				
				return array('status'=>'success', 'zone'=>$zoneInfo['name']);
			} else {
				return $response;
			}
		} catch (\Exception $e) {
			return array('status'=>'error','description'=>$e->getMessage());
		}
	}
	
	/**
	 * If for some reaason a zone is added to WHMCS, but not to ClouDNS this method will add it to ClouDNS.
	 * It can be used for faster zones adding
	 * @param type $zone
	 * @return array
	 */
	public function addNewExistingZone ($zone) {
		try {
			$ns = array();
			$servers = $this->core->Servers->getAvailableServers();
			foreach ($servers as $server) {
				$ns[] = $server['name'];
			}
			$response = $this->core->Zones->addMaster($zone, '1', $ns);
			return $response;
		} catch (\Exception $e) {
			return array('status'=>'error','description'=>$e->getMessage());
		}
	}

	/**
	 * Checks if the zone exists in ClouDNS and here
	 * @param string $zone
	 * @return array
	 */
	public function zoneExists ($zone) {
		$whmcsZone = $this->getZoneByName($zone);
		$cloudnsZone = $this->core->Zones->getZoneInfo($zone);
		
		if (empty($whmcsZone)) {
			return array('status'=>'info', 'description'=>'There is no such DNS zone in the system');
		} elseif (empty($cloudnsZone)) {
			return array('status'=>'error', 'description'=>'There is no such DNS zone with the DNS servers');
		}
		
		return array('status' => 'success');
	}
	
	/**
	 * Checks if a zone is already added in ClouDNS
	 * @param string $zone
	 * @return boolean
	 */
	public function checkForFreeZone ($zone) {
		$response = $this->core->Zones->getZoneInfo($zone);
		// empty $response means that there is no such zone 
		// e.g. it's free and the check can continue
		if (empty($response)) {
			return false;
		} else {
			$tbl_params = array(
				'name' => $zone,
			);
			$zone = $this->core->Database->select('mod_cloudns_zones', $tbl_params);
			if (!empty($zone)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Gets the zones added in the module table here to be displayed in the zones list
	 * @return array
	 */
	public function getZones() {
		try {
			
			$tbl_params = array(
				'serviceid' => $this->params['serviceid']
			);
			$order = 'name';
			$zones = $this->core->Database->select('mod_cloudns_zones', $tbl_params, $order);
			foreach ($zones as &$zone) {
				$zone['ascii'] = $this->core->Helper->getUnicodeName($zone['name']);
			}
			if (empty($zones)) {
				return array('status'=>'error','description'=>'You don\'t have any DNS zones yet.');
			}
			
			return $zones;

		} catch (\Exception $e) {
			return array('status'=>'error','description'=>$e->getMessage());
		}
	}
	
	/**
	 * Count the zones for this service
	 * @return int 
	 */
	public function countUsersZones () {
		try {
			
			$tbl_params = array(
				'serviceid' => $this->params['serviceid']
			);
			
			$count = $this->core->Database->count('mod_cloudns_zones', $tbl_params);
			
			return $count;

		} catch (\Exception $e) {
			return array('status'=>'error','description'=>$e->getMessage());
		}
	}
	
	/**
	 * Checks if there is a zone with that name. It's used above to check if the zone exists in WHMCS
	 * @param string $zone
	 * @return boolean
	 */
	public function getZoneByName ($zone) {
		return $this->core->Database->select('mod_cloudns_zones', array('name'=>$zone));
		
	}
	
	/**
	 * Gets the records of a zone, all of them or only by selected type
	 * @param string $zone
	 * @param string $type
	 * @return array
	 */
	public function getRecords ($zone, $type) {
		if ($type == 'all') {
			return $this->core->Records->getRecords($zone);
		}
		
		$recordsList = $this->core->Records->getRecords($zone);
		
		$records = array();
		foreach ($recordsList as $record) {
			if ($type == $record['type']) {
				$records[] = $record;
			}
		}
		
		return $records;
	}
	
	/**
	 * Gets record by ID for the Edit record page
	 * @param string $zone
	 * @param int $record_id
	 * @return array
	 */
	public function getRecordById ($zone, $record_id) {
		$recordsList = $this->core->Records->getRecords($zone);
		
		if (!isset($recordsList[$record_id])) {
			return array();
		}
		return $recordsList[$record_id];
	}
	
	/**
	 * Gets the record types for this zone type (forward or reverse)
	 * @param string $zone_type
	 * @return array
	 */
	public function getRecordTypes($zone_type) {
		if ($zone_type == 'ipv4' || $zone_type == 'ipv6') {
			$zone_type = 'reverse';
		}
		
		return $this->core->Records->getAvailableRecords($zone_type);
	}
	
	/**
	 * Gets the SOA Primary NS, which is the first server in the list
	 * @return string
	 */
	public function getSOAPrimaryNS () {
		$servers = $this->core->Servers->getAvailableServers();
		
		return $servers[0]['name'];
	}
	
	/**
	 * 
	 * @param string $zone
	 * @return array
	 */
	public function deleteZone ($zone) {
		try {
			$tbl_params = array(
				'serviceid' => $this->params['serviceid'],
				'name' => $zone
			);
			/* we are deleting the zone from the module table */
			$zones = $this->core->Database->delete('mod_cloudns_zones', $tbl_params);

			if (empty($zones)) {
				return array('status'=>'error','description'=>'There is no such zone in your account');
			}

		} catch (\Exception $e) {
			return array('status'=>'error','description'=>$e->getMessage());
		}
		
		/* if everything is okay we can delete the zone from ClouDNS */
		return $this->core->Zones->delete($zone);
	}
	
	public function getAvailableTTL () {
		$ttlValues = $this->core->Records->getAvailableTTL();
		$ttlKeys = array(
			'1 minute', 
			'5 minutes', 
			'15 minutes', 
			'30 minutes', 
			'1 hour', 
			'6 hours', 
			'12 hours', 
			'1 day', 
			'2 days', 
			'3 days', 
			'1 week', 
			'2 weeks', 
			'1 month',
		);
		$ttl = array_combine($ttlKeys, $ttlValues);
		
		return $ttl;
	}
}
